//-------------------------------------------------------------
// CAbout.cs
//
// This implements the object that is responsible for communicating
// "about" information to the MMC.
//
// Its GUID is {01FC9D52-BFD8-4dfc-90C7-56482F48C1C6}
//-------------------------------------------------------------
using System;
using System.Runtime.InteropServices;

namespace Microsoft.SampleMMC
{
[Guid("01FC9D52-BFD8-4dfc-90C7-56482F48C1C6")]
public class CAbout : ISnapinAbout
{
    int m_hMainIcon;         // Handle for the main icon
    int m_hMainBMP16;        // Handle for the 16x16 main bitmap
    int m_hMainBMP32;        // Handle for the 32x32 main bitmap

    //-------------------------------------------------
    // CAbout - Constructor
    //
    // The constructor is responsible for loading the about images
    // that will be displayed in the MMC
    //-------------------------------------------------
    public CAbout()
    {
        int hModule = Marshal.GetHINSTANCE(this.GetType().Module);

        
        m_hMainIcon = LoadIcon(hModule, "IDI_MAIN");

        // Load the main bitmap as a 16x16 bitmap
        m_hMainBMP16 = LoadImage(hModule,
                                 "IDB_MAIN",
                                 IMAGE.BITMAP,
                                 16,
                                 16,
                                 0);
                             

        // Load the main bitmap as a 32x32 bitmap

        m_hMainBMP32 = LoadImage(hModule,
                                 "IDB_MAIN",
                                 IMAGE.BITMAP,
                                 32,
                                 32,
                                 0);
    }// CAbout

    //-------------------------------------------------
    // ~CAbout
    //
    // The destructor is responsible for destroying the
    // images that were loaded in the constructor
    //-------------------------------------------------
    ~CAbout()
    {
        if (m_hMainIcon != 0)
          DestroyIcon(m_hMainIcon);
        if (m_hMainBMP16 != 0)
          DeleteObject(m_hMainBMP16);
        if (m_hMainBMP32 != 0)
          DeleteObject(m_hMainBMP32);
          
    }// ~CAbout
                      
    //-------------------------------------------------
    // GetSnapinDescription
    //
    // This returns the Description of the snapin through
    // a reference parameter
    //-------------------------------------------------
	public void GetSnapinDescription(out int lpDescription)
	{
		lpDescription = Marshal.StringToCoTaskMemUni("A C# sample snapin");
	}// GetSnapinDescription

    //-------------------------------------------------
    // GetProvider
    //
    // This returns the provider of the snapin through
    // a reference parameter
    //-------------------------------------------------
	public void GetProvider(out int pName)
	{
		pName = Marshal.StringToCoTaskMemUni("Microsoft Corporation");
	}// GetProvider

    //-------------------------------------------------
    // GetSnapinVersion
    //
    // This returns the version of the snapin through
    // a reference parameter
    //-------------------------------------------------

	public void GetSnapinVersion(out int lpVersion)
	{
		lpVersion = Marshal.StringToCoTaskMemUni("1.23456");
	}// GetSnapinVersion

    //-------------------------------------------------
    // GetSnapinImage
    //
    // This returns the image of the snapin through
    // a reference parameter that is used in the About
    // page that is referenced from the Add/Remove snapin
    // page.
    //-------------------------------------------------
	public void GetSnapinImage(out int hAppIcon)
	{
       hAppIcon=m_hMainIcon;
	}// GetSnapinImage

    //-------------------------------------------------
    // GetStaticFolderImage
    //
    // This returns the Description of the snapin through
    // a reference parameter
    //-------------------------------------------------
	public void GetStaticFolderImage(out int hSmallImage, out int hSmallImageOpen, out int hLargeImage, out int cMask)
	{
	    hSmallImage=hSmallImageOpen=m_hMainBMP16;
	    hLargeImage=m_hMainBMP32;
        // The color mask on these bitmaps is pure white.... it could be different
        // with different bitmaps
	    cMask=0x00FFFFFF;
	}// GetStaticFolderImage

	//-------------------------------------------------
    // We need to import the Win32 API calls used to deal with
    // image loading.
    //-------------------------------------------------
    [DllImport("user32.dll")]
	public static extern int LoadIcon(int hinst, String name);
	[DllImport("user32.dll")]
    public static extern int DestroyIcon(int hIcon);
	[DllImport("user32.dll")]
	public static extern int LoadImage(int hinst, String lpszName,  uint uType, int cxDesired, int cyDesired, uint fuLoad);
    [DllImport("gdi32.dll")]
    public static extern int DeleteObject(int hObject);
}// class CAbout
}//namespace Microsoft.SampleMMC
