//==========================================================================
//  File:		host.cs
//
//  Summary:	This file implements "type resolution" sample.  This sample
//              demonstrates how hosts can participate in the type resolution
//              process by supplying and event handler that returns an assembly
//              containing the requested type.
//			 
//  Classes:	Host
//
//--------------------------------------------------------------------------
//  This file is part of the Microsoft NGWS Samples.
//
//  Copyright (C) 2000 Microsoft Corporation.  All rights reserved.
//==========================================================================
using System;
using System.Reflection;
using System.Reflection.Emit;
using System.Threading;
using System.Runtime.Remoting;


public class Host 
{

	private Assembly EmitAssembly()
	{
	
		//
		// Create an assembly name
		//
		AssemblyName assemblyName = new AssemblyName();
		assemblyName.Name = "SecondAssembly";

		//
		// Create a new assembly with one module
		//
        AssemblyBuilder newAssembly = Thread.GetDomain().DefineDynamicAssembly(assemblyName, AssemblyBuilderAccess.Run);
		ModuleBuilder newModule = newAssembly.DefineDynamicModule("MyModule");

		//		
        //  Define a public class named "YourType" in the assembly.
		//			
        TypeBuilder myType = newModule.DefineType("Samples.TypeResolve.Second.YourType", TypeAttributes.Public);

		//
		// Define a method on the type to call
		//
		MethodBuilder simpleMethod = myType.DefineMethod("SimpleMethod", MethodAttributes.Public, null, null);

		ILGenerator il = simpleMethod.GetILGenerator();
		il.EmitWriteLine("Method called in second assembly");
		il.Emit(OpCodes.Ret);

		//
		// Finish it off ...
		//
		myType.CreateType();
  

      	return newAssembly;
	}

	private Assembly TLoadHandler(System.Object sender, EventArgs e)
	{
		Console.WriteLine ("Event Handler Called");
		return EmitAssembly();
	}

	public static void Main(string[] args)
	{
		ResolveEventHandler trh;
		Host host = new Host();

		ObjectHandle oh = Thread.GetDomain().CreateInstance("first.dll","Samples.TypeResolve.First.MyType"); 
		Type mt = oh.Unwrap().GetType();

		Assembly firstAssembly = mt.Module.Assembly;

		
		//
		// Hook up the event handler
		//
		trh= new ResolveEventHandler ( host.TLoadHandler);
		firstAssembly.AddOnTypeResolve(trh);

		//
		// Call a method -- this will result in an unresolved type,
		// which will trigger the event handler
		//
		MethodInfo myMethod = mt.GetMethod("CallSecond");
			if (myMethod==null){
				Console.WriteLine("Couldn't find method to call");
			} else {
				Object o = Activator.CreateInstance(mt);
				myMethod.Invoke(o,null);
			}
	}

}